/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.core;

import org.apache.axiom.om.impl.llom.util.AXIOMUtil;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.clustering.ClusteringConstants;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.commons.collections.BidiMap;
import org.apache.commons.collections.bidimap.TreeBidiMap;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.core.persistence.PersistenceManager;
import org.wso2.carbon.core.transports.CommonTransportLoader;
import org.wso2.carbon.core.util.ClusteringUtil;
import org.wso2.carbon.core.util.HouseKeepingTask;
import org.wso2.carbon.core.util.SystemRestarter;
import org.wso2.carbon.registry.core.Resource;
import org.wso2.carbon.utils.CarbonUtils;
import org.wso2.carbon.utils.ConfigurationContextService;
import org.wso2.carbon.utils.Controllable;
import org.wso2.carbon.utils.FileManipulator;
import org.wso2.carbon.utils.Monitor;
import org.wso2.carbon.utils.ServerConfiguration;
import org.wso2.carbon.utils.ServerConfigurationException;
import org.wso2.carbon.utils.ServerConstants;
import org.wso2.carbon.utils.ServerException;
import org.wso2.carbon.utils.NetworkUtils;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.Iterator;
import java.net.SocketException;

/**
 * TODO: please use i18n for error messages.
 */
public class StartupServlet extends HttpServlet implements Controllable {
    private static final Log log = LogFactory.getLog(StartupServlet.class);
    private static final long serialVersionUID = 4517849288538613640L;
    private final String CLIENT_REPOSITORY_LOCATION = "Axis2Config.ClientRepositoryLocation";
    private final String CLIENT_AXIS2_XML_LOCATION = "Axis2Config.clientAxis2XmlLocation";

    protected ServerManager serverManager;
    protected String serverName;

    private ServletConfig servletConfig;
    private String carbonHome;
    private ServerConfiguration serverConfig;
    private Thread shutdownHook;
    private BundleContext bundleContext;
    public boolean isEmbedEnv = false;

    public StartupServlet(BundleContext bundleContext) {
        super();
        this.bundleContext = bundleContext;
    }

    public void init(ServletConfig servletConfig) throws ServletException {
        if(System.getProperty(CarbonConstants.START_TIME) == null){
            System.setProperty(CarbonConstants.START_TIME, System.currentTimeMillis() + "");
        }
        this.servletConfig = servletConfig;

        // Expanding of web content within AAR file
        try {
            System.setProperty(ServerConstants.WEB_RESOURCE_LOCATION,
                               servletConfig.getServletContext().getRealPath("/wservices"));
        } catch (Exception e) {
            log.warn("server .war is not expanded. Web content within AAR file not supported.");
        }

        //TODO : all system properties must be handle carefully; hack to get this working
        // when the module is an unpacked war file,
        // we can set the web location path in the deployment engine.
        // This will let us
        String temp = System.getProperty(ServerConstants.STANDALONE_MODE);
        boolean isStandalone = false;
        if (temp != null) {
            temp = temp.trim();
            isStandalone = temp.equals("true");
        }

        serverConfig = ServerConfiguration.getInstance();
        serverManager = ServerManager.getInstance();
        serverManager.setBundleContext(bundleContext);

        if (!isStandalone) {
            //This is the mode where Carbon starts in an standalone env
            // This is the first impl. Fallbakc to defautls from bundle context
            //TODO this a just a temprary solution please fix this using OSGi ManageService/ManageServiceFactory
            isEmbedEnv = false;
            String carbonHomeFromEnv = System.getenv("CARBON_HOME");

            if (carbonHomeFromEnv == null) {
                throw new ServletException(
                        "CARBON_HOME environment variable is not set. Could not initialize " +
                        "server configuration");
            }

            carbonHome = carbonHomeFromEnv;
            
            System.setProperty(ServerConstants.AXIS2_HOME, carbonHome);
            System.setProperty(ServerConstants.CARBON_HOME, carbonHome);
            try {
                System.setProperty(ServerConstants.LOCAL_IP_ADDRESS, NetworkUtils.getLocalHostname());
            } catch (SocketException ignored) {
            }
            try {
                File carbonXML = new File(CarbonUtils.getServerXml());
                InputStream inSXml = new FileInputStream(carbonXML);
                serverConfig.forceInit(inSXml);
            } catch (IOException e) {
                throw new ServletException(e);
            } catch (ServerConfigurationException e) {
                throw new ServletException(e);
            }

        } else {
            //TODO only temp
            isEmbedEnv = true;
            /*carbonHome = webpath + File.separator + "WEB-INF";
            System.out.println("StartupServlet Starts using Carbon Home : " + carbonHome);*/
            //Setting the Carbon home
            carbonHome = System.getProperty(ServerConstants.CARBON_HOME);
            if (carbonHome == null) {
                String msg = ServerConstants.CARBON_HOME +
                             " System property has not been";
                log.fatal(msg);
                log.fatal(serverName + " startup failed.");
                throw new ServletException(msg);
            }

            System.setProperty(ServerConstants.AXIS2_HOME, carbonHome);
            System.setProperty(ServerConstants.CARBON_HOME, carbonHome);
            try {
                System.setProperty(ServerConstants.LOCAL_IP_ADDRESS, NetworkUtils.getLocalHostname());
            } catch (SocketException ignored) {
            }

            try {
                File carbonXML = new File(CarbonUtils.getServerXml());
                InputStream inSXml = new FileInputStream(carbonXML);
                serverConfig.forceInit(inSXml);
            } catch (ServerConfigurationException e) {
                String msg = "Could not initialize server configuration";
                log.fatal(msg);
                throw new ServletException(msg);
            } catch (FileNotFoundException e) {
                throw new ServletException(e);
            }
        }
        serverName = serverConfig.getFirstProperty("Name");
        try {
            start();
        } catch (ServerException e) {
            e.printStackTrace();
            String msg = "Could not start " + serverName;
            log.fatal(msg, e);
            throw new ServletException(msg);
        }

        //Registering ServerConfiguration as an OSGi service
        bundleContext.registerService(ServerConfiguration.class.getName(),serverConfig,null);
        
        // Populates transport properties from the registry.
        populateTransportParams();

        // Listeners should be initialize before ServerInfo processing. 
        serverManager.startListenerManager();

        // Print information about the system
        printInfo();

        // Initialize the cluster
        try {
            ClusteringUtil.enableClustering(serverManager.getConfigContext());
        } catch (AxisFault e) {
            String msg = "Cannot initialize cluster";
            log.error(msg, e);
            throw new ServletException(msg, e);
        }
    }

    public void start() throws ServerException {

        String hostName = ServerConfiguration.getInstance().getFirstProperty("ClusteringHostName");
        if (System.getProperty(ClusteringConstants.LOCAL_IP_ADDRESS) == null &&
            hostName != null && hostName.trim().length() != 0) {
            System.setProperty(ClusteringConstants.LOCAL_IP_ADDRESS, hostName);
        }
        
        serverManager.serverWorkDir =
                new File(serverConfig.getFirstProperty("WorkDirectory")).getAbsolutePath();
        System.setProperty("axis2.work.dir", serverManager.serverWorkDir);

        if (System.getProperty("axis2.repo") != null) {
            serverManager.axis2RepoLocation = System.getProperty("axis2.repo");
            serverConfig.setConfigurationProperty(ServerConfiguration.AXIS2_CONFIG_REPO_LOCATION,
                                                  serverManager.axis2RepoLocation);
        } else {
            if (isEmbedEnv) {
                serverManager.axis2RepoLocation =
                        serverConfig.getFirstProperty(ServerConfiguration.AXIS2_CONFIG_REPO_LOCATION);
            } else {
                serverManager.axis2RepoLocation = System
                        .getProperty(ServerConstants.AXIS2_HOME) + File.separator + "repository" + File.separator;
            }
        }


        if (!serverManager.axis2RepoLocation.endsWith("/")) {
            serverConfig.setConfigurationProperty(ServerConfiguration.AXIS2_CONFIG_REPO_LOCATION,
                                                  serverManager.axis2RepoLocation + "/");
            serverManager.axis2RepoLocation = serverManager.axis2RepoLocation + "/";
        }

        try {
            serverManager.start();
            addShutdownHook();
            ConfigurationContext configCtx = serverManager.getConfigContext();
            registerHouseKeepingTask(configCtx);
            servletConfig.getServletContext().setAttribute(ServerConstants.CONFIGURATION_CONTEXT,
                                                           configCtx);
            configCtx.setProperty(Constants.CONTAINER_MANAGED, "true");
            configCtx.setProperty(ServerConstants.WORK_DIR, serverManager.serverWorkDir);
            //This is used inside the sever-admin component.
            configCtx.setProperty(ServerConstants.CARBON_INSTANCE, this);

//            addConfigServiceListeners(configCtx);

            String clientRepositoryLocation =
                    serverConfig.getFirstProperty(CLIENT_REPOSITORY_LOCATION);
            String clientAxis2XmlLocationn = serverConfig.getFirstProperty(CLIENT_AXIS2_XML_LOCATION);
            //Creating the Client side configuration context
            ConfigurationContext clientConfigContext =
                    ConfigurationContextFactory.createConfigurationContextFromFileSystem(
                            clientRepositoryLocation, clientAxis2XmlLocationn);
            registerHouseKeepingTask(clientConfigContext);
            clientConfigContext.setProperty(ServerConstants.WORK_DIR, serverManager.serverWorkDir);

            //TOa house keeping taskDO add this map to a house keeping task
            //Adding FILE_RESOURCE_MAP
            Object property = new TreeBidiMap();
            clientConfigContext.setProperty(ServerConstants.FILE_RESOURCE_MAP, property);
            //Registering the configuration contexts as a service.
            bundleContext.registerService(ConfigurationContextService.class.getName(),
                    new ConfigurationContextService(configCtx,clientConfigContext), null);

        } catch (Exception e) {
            throw new ServerException(serverName + " startup failed.", e);
        }
    }

    private void registerHouseKeepingTask(ConfigurationContext configurationContext) {
        if (Boolean.valueOf(serverConfig.getFirstProperty("HouseKeeping.AutoStart"))) {
            Timer houseKeepingTimer = new Timer();
            long houseKeepingInterval =
                    Long.parseLong(serverConfig.
                            getFirstProperty("HouseKeeping.Interval")) * 60 * 1000;
            Object property =
                    configurationContext.getProperty(ServerConstants.FILE_RESOURCE_MAP);
            if (property == null) {
                property = new TreeBidiMap();
                configurationContext.setProperty(ServerConstants.FILE_RESOURCE_MAP, property);
            }
            houseKeepingTimer
                    .scheduleAtFixedRate(new HouseKeepingTask(serverManager.serverWorkDir,
                                                              (BidiMap) property),
                                         houseKeepingInterval,
                                         houseKeepingInterval);
        }
    }

//    private void addConfigServiceListeners(ConfigurationContext configCtx) throws InvalidSyntaxException {
//        bundleContext.addServiceListener(new Axis2ConfigServiceListener(configCtx, bundleContext),
//                                         "("+ CarbonConstants.AXIS2_CONFIG_SERVICE +"=*)");
//    }

    private void printInfo() {
        if (System.getProperty(ServerConstants.REPO_WRITE_MODE, "false").equals("false")) {
            System.out.println();
            log.info("HTTPS port             : " + System.getProperty(ServerConstants.HTTPS_PORT));
            log.info("HTTP port              : " + System.getProperty(ServerConstants.HTTP_PORT));
            System.out.println();
        }
        long startTime = Long.parseLong(System.getProperty(CarbonConstants.START_TIME));
        long startupTime = (System.currentTimeMillis() - startTime) / 1000;
        log.info("WSO2 Carbon started in " + startupTime + " sec");
        System.getProperties().remove(CarbonConstants.START_TIME);
    }

    private void addShutdownHook() {
        if (shutdownHook != null) {
            return;
        }
        shutdownHook = new Thread() {
            public void run() {
                log.info("Shutting down " + serverName + "...");
                try {
                    cleanupSystem();
                    try {
                        serverManager.stopListenerManager();
                    } catch (AxisFault e) {
                        String msg = "Error occurred while stopping Axis2 Listener Manager";
                        log.error(msg, e);
                        throw new ServerException(msg, e);
                    }
                    log.info("Shutdown complete");
                    log.info("Halting JVM");
                } catch (Exception e) {
                    log.warn("Error occurred while shutting down WSO2 WSAS", e);
                }
            }
        };
        Runtime.getRuntime().addShutdownHook(shutdownHook);
    }

    public void setAxis2RepoLocation(String axis2RepoLocation) {
        serverManager.axis2RepoLocation = axis2RepoLocation;
    }

    public void setMonitor(Monitor monitor) {
        serverManager.monitor = monitor;
    }

    public void startServer() throws ServerException {
        try {
            init(this.servletConfig);
            reinitializeServlets(this.servletConfig.getServletContext());
            try {
                ServerStatus.setServerRunning();
            } catch (AxisFault e) {
                String msg = "Cannot set server to running mode";
                log.error(msg, e);
            }
        } catch (ServletException e) {
            String msg = "Cannot start server";
            log.fatal(msg, e);
            throw new ServerException(msg, e);
        }
    }

    public void stopListeners() throws ServerException {
    }

    public void restart() {
        SystemRestarter st = new SystemRestarter(false, serverManager.getConfigContext());
        cleanupSystem();
        new Thread(st).start();
        Runtime.getRuntime().removeShutdownHook(shutdownHook);
        serverManager.stopJmxService();
    }

    public void restartGracefully() {
        try {
            ServerStatus.setServerRestarting();
        } catch (AxisFault e) {
            String msg = "Cannot set server to restarting mode";
            log.error(msg, e);
        }
        SystemRestarter st = new SystemRestarter(true, serverManager.getConfigContext());
        cleanupSystem();
        new Thread(st).start();
        Runtime.getRuntime().removeShutdownHook(shutdownHook);
        serverManager.stopJmxService();
    }

    public void shutdown() {
        try {
            ServerStatus.setServerShuttingDown();
        } catch (AxisFault e) {
            String msg = "Cannot set server to shutdown mode";
            log.error(msg, e);
        }
        Runtime.getRuntime().removeShutdownHook(shutdownHook);
        System.exit(0);
    }

    public void shutdownGracefully() {
        try {
            ServerStatus.setServerShuttingDown();
            log.info("Gracefully shutting down " + serverName + "...");
            new ServerManagement(this.getConfigurationContext().
                    getAxisConfiguration().getTransportsIn()).startMaintenance();
        } catch (Exception e) {
            String msg = "Cannot set server to shutdown mode";
            log.error(msg, e);
        }
        System.exit(0);
    }

    public Thread getShutdownHook() {
        return shutdownHook;
    }

    public void setConfigurationContext(ConfigurationContext newConfigCtx) throws
                                                                           ServerException {
        try {
            // Stop the ListenerManager
            try {
                serverManager.stopListenerManager();
            } catch (AxisFault axisFault) {
                throw new ServerException(axisFault);
            }

            // Now we switch to the new Config ctx
            serverManager.setConfigContext(newConfigCtx);

            // Disable admin services, if needed
            if (!CarbonUtils.isAdminConsoleEnabled()) {
                newConfigCtx.getAxisConfiguration().
                        removeServiceGroup(ServerConstants.ADMIN_SERVICE_GROUP);
            }

            ServletContext servletContext = servletConfig.getServletContext();
            servletContext.removeAttribute(ServerConstants.CONFIGURATION_CONTEXT);
            servletContext.setAttribute(ServerConstants.CONFIGURATION_CONTEXT,
                                        newConfigCtx);
            reinitializeServlets(servletContext);

            // Need to enable clustering on the new configurationContext
            try {
                ClusteringUtil.enableClustering(newConfigCtx);
            } catch (AxisFault axisFault) {
                throw new ServerException(axisFault);
            }

            // Start the ListenerManager
            serverManager.startListenerManager();
        } catch (Exception e) {
            String msg = "Cannot set ConfigurationContext";
            log.fatal(msg, e);
            throw new ServerException(msg, e);
        }
    }

    public ConfigurationContext getConfigurationContext() {
        return serverManager.getConfigContext();
    }

    private void reinitializeServlets(ServletContext servletContext) throws ServletException {
        // Reinitialize all of the servlets, for this puprpose,
        // the servlets should have added themselves
        // as ServletContext attributes in the previous init call
        Enumeration attributeNames = servletContext.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object attrib = servletContext.getAttribute((String) attributeNames.nextElement());
            if (attrib instanceof HttpServlet) {
                ((HttpServlet) attrib).init();
            } /*else if (attrib instanceof AdminUIServletFilter) {
                AdminUIServletFilter adminUIServletFilter = (AdminUIServletFilter) attrib;
                adminUIServletFilter.init();
            }*/
        }
    }
    
    /**
     * Populates transport properties from the registry.
     */
	private void populateTransportParams() throws ServletException {

		TransportInDescription description = null;
		PersistenceManager pm = null;
		List params = null;
		Resource resource = null;
		Resource resourceElement = null;
		Resource availTransports = null;
		String availTransportsContent = null;
		String[] transports = null;
		HashMap transportIns = null;
		Set<String> transportNames = null;
		AxisConfiguration axisConfiguration = null;
		ConfigurationContext configContext = null;
		String name = null;

		try {
			configContext = serverManager.getConfigContext();
			axisConfiguration = configContext.getAxisConfiguration();

			pm = new PersistenceManager(axisConfiguration);
			// This will return all the available transports -what is defined in
			// axis2.xml as well as what comes from transport bundles.
			availTransports = pm.getAvailableTransports();
			if (availTransports != null) {
				boolean foundTransportNotFromAxis2Xml = false;
				CommonTransportLoader transportLoader = null;
				availTransportsContent = new String((byte[]) availTransports.getContent());
				// All available transport names are stored in registry as a
				// comma separated string
				transports = availTransportsContent.split(",");
				availTransports.discard();

				for (int i = 0; i < transports.length; i++) {
					if (axisConfiguration.getTransportIn(transports[i]) == null) {
						// All configuration files from transport bundles will
						// adhere to the convention -[transport_name]-transports.xml
						// We need to add the transports to the AxisConfiguration - which comes from
						// transport bundles and which are not defined in axis2.xml
						TransportInDescription transportInDesc = null;
						transportLoader = new CommonTransportLoader(axisConfiguration,
								transports[i].trim() + "-transports.xml");
						transportInDesc = transportLoader.readTransportConfiguration();
						if (transportInDesc != null) {
							foundTransportNotFromAxis2Xml = true;
							axisConfiguration.addTransportIn(transportInDesc);
						}
					}
				}

				if (foundTransportNotFromAxis2Xml) {
					transportLoader.enableTransport();
				}
			}

			transportIns = axisConfiguration.getTransportsIn();
			transportNames = transportIns.keySet();

			for (Iterator<String> namesIterator = transportNames.iterator(); namesIterator
					.hasNext();) {
				description = axisConfiguration.getTransportIn(namesIterator.next());
				params = description.getParameters();
				name = description.getName();

				if (log.isDebugEnabled()) {
					log.debug("Activating transport for " + name);
				}

				for (Iterator<Parameter> paramIterator = params.iterator(); paramIterator.hasNext();) {
					Parameter parameter = paramIterator.next();

					String key = parameter.getName();
					String content = null;
					String elementContent = null;
					Parameter param = null;

					resource = pm.getTransportParam(name, key);
					resourceElement = pm.getTransportParam(name, key + ".element");

					// We need to attend this later - right now we don't let
					// users edit the port. So we read it from base configuration.
					if (!"port".equals(key)) {
						param = description.getParameter(key);
						if (resource != null) {
							content = new String((byte[]) resource.getContent());
							try {
								param.setValue(AXIOMUtil.stringToOM(content));
							} catch (Exception e) {
								param.setValue(content);
							}
						}

						if (resourceElement != null) {
							elementContent = new String((byte[]) resourceElement.getContent());
							param.setParameterElement(AXIOMUtil.stringToOM(elementContent));
						}
					}

					if (resourceElement != null) {
						resourceElement.discard();
					}
					if (resource != null) {
						resource.discard();
					}
				}
			}
		} catch (Exception e) {
			log.error("Error while populating transport properties from the registry.", e);
			throw new ServletException(
					"Error while populating transport properties from the registry", e);
		}
	}
	
    public void destroy() {
    }

    private void cleanupSystem() {
        log.info("Cleaning up system...");
        new FileManipulator().deleteDir(new File(carbonHome + File.separator +
                                                 serverConfig.
                                                         getFirstProperty("WorkDirectory")));
        ConfigurationContext configContext = serverManager.getConfigContext();
        if (configContext != null) {
            Object property = configContext.getProperty(ServerConstants.FILE_RESOURCE_MAP);
            if (property != null) {
                ((Map) property).clear();
            }
        }
    }
}
