/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.core.internal;

import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.AxisFault;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.http.HttpContext;
import org.osgi.service.http.HttpService;
import org.osgi.util.tracker.ServiceTracker;
import org.osgi.util.tracker.ServiceTrackerCustomizer;
import org.wso2.carbon.core.StartupServlet;
import org.wso2.carbon.core.security.CarbonJMXAuthenticator;
import org.wso2.carbon.core.transports.CarbonServlet;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.utils.ConfigurationContextService;
import org.wso2.carbon.utils.ServerConfiguration;

/**
 *
 */
public class Activator implements BundleActivator, ServiceTrackerCustomizer {

    private static Log log = LogFactory.getLog(Activator.class);
    private BundleContext bundleContext;
   
    public void start(BundleContext context) throws Exception {
        this.bundleContext = context;
        
        ServiceReference reference = context.getServiceReference(HttpService.class.getName());
        if (reference == null) {
            String msg = "Before activating Carbon Core bundle, an instance of "
                    + HttpService.class.getName() + " should be in existance";
            log.error(msg);
            throw new Exception(msg);
        }
        HttpService httpService = (HttpService) context.getService(reference);
        HttpContext defaultHttpContext = httpService.createDefaultHttpContext();

        StartupServlet startupServlet = new StartupServlet(context);
        httpService.registerServlet("/startup_servlet", startupServlet, null, defaultHttpContext);

        CarbonServlet carbonServlet = new CarbonServlet();
        ConfigurationContext configContext = startupServlet.getConfigurationContext();

        String servicePath = "/services";
        if (configContext != null) {
            String path = configContext.getServicePath();
            if (path != null) {
                servicePath = path.trim();
            }
        }
        if (!servicePath.startsWith("/")) {
            servicePath = "/" + servicePath;
        }
        httpService.registerServlet(servicePath, carbonServlet, null, defaultHttpContext);

        ServiceReference[] serviceRefs =
                context.getServiceReferences(UserRealm.class.getName(), "(RealmGenre=Delegating)");
        ServiceTracker dlgtRealmTracker = new ServiceTracker(context, serviceRefs[0],  this);
        dlgtRealmTracker.open();
        if (dlgtRealmTracker.getService() != null) {
            UserRealm realm = (UserRealm) dlgtRealmTracker.getService();
            CarbonJMXAuthenticator.setUserRealm(realm);
        }


        if (log.isDebugEnabled()) {
            log.debug("Carbon Core is activated");
        }
    }

    public void stop(BundleContext context) throws Exception {
        ServiceReference reference = context.getServiceReference(HttpService.class.getName());
        if (reference == null) {
            String msg = "Before stopping Carbon Core, an instance of "
                    + HttpService.class.getName() + " should be in existence";
            log.error(msg);
            throw new Exception(msg);
        }
        HttpService httpService = (HttpService) context.getService(reference);
        httpService.unregister("/startup_servlet");
        httpService.unregister("/services");
    }

    public Object addingService(ServiceReference serviceReference) {
        Object serviceObject;
        synchronized (Activator.class) {
            serviceObject = bundleContext.getService(serviceReference);
            if (serviceObject instanceof UserRealm) {
                UserRealm realm = (UserRealm) serviceObject;
                CarbonJMXAuthenticator.setUserRealm(realm);
            }
        }
        return serviceObject;
    }

    public void modifiedService(ServiceReference serviceReference, Object o) {
    }

    public void removedService(ServiceReference serviceReference, Object o) {
    }
}
