package org.wso2.carbon.core.transports;

import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.axis2.deployment.DeploymentConstants;
import org.apache.axis2.deployment.DeploymentErrorMsgs;
import org.apache.axis2.deployment.DeploymentException;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.description.ParameterInclude;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.i18n.Messages;
import org.apache.axis2.transport.TransportListener;
import org.apache.axis2.util.Loader;

public class TransportBuilder implements DeploymentConstants {

	/**
	 * 
	 * @param transportReivers
	 * @return
	 * @throws DeploymentException
	 */
	protected TransportInDescription[] processTransportReceivers(Iterator transportReivers)
			throws DeploymentException {

		ArrayList<TransportInDescription> transportIns = new ArrayList<TransportInDescription>();

		while (transportReivers.hasNext()) {
			TransportInDescription transportIN;
			OMElement transport = (OMElement) transportReivers.next();
			// getting transport Name
			OMAttribute trsName = transport.getAttribute(new QName(ATTRIBUTE_NAME));
			if (trsName != null) {
				String name = trsName.getAttributeValue();
				transportIN = new TransportInDescription(name);
				// transport implementation class
				OMAttribute trsClas = transport.getAttribute(new QName(TAG_CLASS_NAME));
				if (trsClas != null) {
					try {
						String clasName = trsClas.getAttributeValue();
						Class receiverClass;
						receiverClass = Loader.loadClass(clasName);
						TransportListener receiver = (TransportListener) receiverClass
								.newInstance();
						transportIN.setReceiver(receiver);
					} catch (NoClassDefFoundError e) {
						// TODO: throw new DeploymentException(e);
					} catch (ClassNotFoundException e) {
						throw new DeploymentException(e);
					} catch (IllegalAccessException e) {
						throw new DeploymentException(e);
					} catch (InstantiationException e) {
						throw new DeploymentException(e);
					}
				}
				try {
					Iterator itr = transport.getChildrenWithName(new QName(TAG_PARAMETER));
					processParameters(itr, transportIN);
					transportIns.add(transportIN);
				} catch (AxisFault axisFault) {
					throw new DeploymentException(axisFault);
				}
			}
		}

		return transportIns.toArray(new TransportInDescription[transportIns.size()]);
	}

	/**
	 * 
	 * @param parameters
	 * @param parameterInclude
	 * @throws DeploymentException
	 */
	private void processParameters(Iterator parameters, ParameterInclude parameterInclude)
			throws DeploymentException {
		while (parameters.hasNext()) {
			// this is to check whether some one has locked the parameter at the
			// top level
			OMElement parameterElement = (OMElement) parameters.next();
			Parameter parameter = new Parameter();
			// setting parameterElement
			parameter.setParameterElement(parameterElement);
			// setting parameter Name
			OMAttribute paramName = parameterElement.getAttribute(new QName(ATTRIBUTE_NAME));
			if (paramName == null) {
				throw new DeploymentException(Messages.getMessage(
						DeploymentErrorMsgs.BAD_PARAMETER_ARGUMENT, parameterElement.toString()));
			}
			parameter.setName(paramName.getAttributeValue());
			// setting parameter Value (the child element of the parameter)
			OMElement paramValue = parameterElement.getFirstElement();
			if (paramValue != null) {
				parameter.setValue(parameterElement);
				parameter.setParameterType(Parameter.OM_PARAMETER);
			} else {
				String paratextValue = parameterElement.getText();

				parameter.setValue(paratextValue);
				parameter.setParameterType(Parameter.TEXT_PARAMETER);
			}
			// setting locking attribute
			OMAttribute paramLocked = parameterElement.getAttribute(new QName(ATTRIBUTE_LOCKED));

			if (paramLocked != null) {
				String lockedValue = paramLocked.getAttributeValue();
				if (BOOLEAN_TRUE.equals(lockedValue)) {
					parameter.setLocked(true);
				} else {
					parameter.setLocked(false);
				}
			}
			try {
				parameterInclude.addParameter(parameter);
			} catch (AxisFault axisFault) {
				throw new DeploymentException(axisFault);
			}
		}
	}
}