package org.wso2.carbon.registry.app;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.registry.core.CarbonRegistry;
import org.wso2.carbon.registry.core.ChrootRegistry;
import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.registry.core.session.UserRegistry;
import org.wso2.carbon.user.core.UserRealm;

import java.net.MalformedURLException;

public class RemoteRegistryService implements RegistryService{

    private static final Log log = LogFactory.getLog(RemoteRegistryService.class);
    private Registry registry;
    private UserRealm userRealm;
    private String url;

    public RemoteRegistryService (String url,String username,String password, UserRealm userRealm, String chroot) throws RegistryException {
        try {
            RemoteRegistry remote = new RemoteRegistry(url, username, password);

            //Hack to authenticate the user with the remote registry as remote regitry
            //doesn't provide a way to log in
            remote.get("/");

            //If chroot option is provided, create a ChrootRegistry
            if (chroot != null ) {
                registry = new ChrootRegistry(remote,chroot);
            } else {
                registry = remote;
            }

            this.url = url;
            this.userRealm = userRealm;

        } catch (MalformedURLException e) {
            log.fatal("Registry URL is malformed, Registry configuration must be invalid",e);
            throw new RegistryException("URL is malformed");
        } catch (Exception e) {
            log.fatal("Error initializing the remote registry, Registry configuration must be invalid",e);
            throw new RegistryException("Error initializing the remote registry");
        }
    }

    public UserRegistry getUserRegistry() throws RegistryException {
        return new CarbonRegistry(RegistryConstants.ANONYMOUS_USER, registry, userRealm);
    }

    public UserRegistry getSystemRegistry() throws RegistryException {
        return new CarbonRegistry(RegistryConstants.SYSTEM_USER, registry, userRealm);
    }

    public UserRegistry getUserRegistry(String username, String password) throws RegistryException {
        try {

            RemoteRegistry userRemote = new RemoteRegistry(url, username, password);
            return new CarbonRegistry(username, userRemote, userRealm);

        } catch (MalformedURLException e) {
            log.fatal("Registry URL is malformed, Registry configuration must be invalid",e);
            throw new RegistryException("URL is malformed");
        } catch (Exception e) {
            log.fatal("Error initializing the remote registry, User credentials must be invalid",e);
            throw new RegistryException("Error initializing the remote registry");
        }
    }

    public UserRegistry getUserRegistry(String userName) throws RegistryException {
        return new UserRegistry(userName, registry, userRealm);
    }
}
