package org.wso2.carbon.registry.app;

import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.registry.core.session.UserRegistry;
import org.wso2.carbon.registry.core.jdbc.EmbeddedRegistry;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.apache.axis2.context.MessageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.util.tracker.ServiceTracker;
import org.apache.axiom.om.util.Base64;

import javax.servlet.http.HttpServletRequest;

public class Utils {
    private static final Log log = LogFactory.getLog(Utils.class);

    private static ServiceTracker registryTracker;

    public static synchronized void setRegistryTracker(ServiceTracker tracker) {
        registryTracker = tracker;
    }

    public static Registry getRegistry() throws RegistryException {

        MessageContext messageContext = MessageContext.getCurrentMessageContext();
        if (messageContext == null) {
            String msg = "Could not get the user's Registry session. Message context not found.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        HttpServletRequest request =
                (HttpServletRequest) messageContext.getProperty("transport.http.servletRequest");

        Registry registry =
                (Registry) request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);

        if (registry == null) {

            String msg = "User's Registry instance is not found. " +
                    "Creating a anonymous Registry instance for the user.";
            if(log.isDebugEnabled()) {
                log.debug(msg);
            }

            EmbeddedRegistry embeddedRegistry = (EmbeddedRegistry) registryTracker.getService();
            registry = embeddedRegistry.getUserRegistry(RegistryConstants.ADMIN_USER, RegistryConstants.ADMIN_PASSWORD);
            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, registry);
        }

        return registry;
    }

    public static UserRegistry getRegistry(HttpServletRequest request) throws RegistryException {

        UserRegistry registry =
                (UserRegistry) request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);

	if (registry == null) {
 
             if (!request.getHeader("Authorization").equals(null)) {
                 String auth = request.getHeader("Authorization");
                 if (auth != null && auth.length() > 0) {
                     String[] aParts = auth.trim().split(" ");
                     if (aParts.length == 2) {
                         String credentials = aParts[1];
                         String decodedCredentials = new String(Base64.decode(credentials));
                         String[] cParts = decodedCredentials.trim().split(":");
                         if (cParts.length == 2) {
                             String userName = cParts[0];
                             String password = cParts[1];
 
                             EmbeddedRegistry embeddedRegistry = (EmbeddedRegistry) registryTracker.getService();
                             registry = embeddedRegistry.getUserRegistry(userName, password);
                             request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, registry);
                         }
                     }
                 }
             }
        }
        if (registry == null) {

            String msg = "User's Registry instance is not found. " +
                    "Creating a anonymous Registry instance for the user.";
            if(log.isDebugEnabled()) {
                log.debug(msg);
            }

            EmbeddedRegistry embeddedRegistry = (EmbeddedRegistry) registryTracker.getService();
            registry = embeddedRegistry.getUserRegistry();
            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, registry);
        }

        return registry;
    }
}
