/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc.realm;

import org.wso2.carbon.user.core.*;
import org.wso2.carbon.user.core.def.DefaultRealm;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;
import org.wso2.carbon.user.core.dbcreators.HSQLDatabaseCreator;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.sql.SQLException;

public class InMemoryRegistryRealm implements UserRealm {

    private static final Log log = LogFactory.getLog(InMemoryRegistryRealm.class);

    private UserRealm userRealm;

    public InMemoryRegistryRealm() throws RegistryException {

        // create an inmemory realm
        try {
            Class clazz = Class.forName("org.hsqldb.jdbcDriver");
        } catch (ClassNotFoundException e) {
            String msg = "HSQL database driver is not available in the class path. " +
                    "Could not create the database for the user manager.";
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        BasicDataSource ds = new BasicDataSource();
        ds.setUrl("jdbc:hsqldb:mem:aname");
        ds.setDriverClassName("org.hsqldb.jdbcDriver");

        HSQLDatabaseCreator creator = new HSQLDatabaseCreator(ds);
        try {
            creator.createDefaultDatabaseTables();
        } catch (SQLException e) {
            String msg = "Failed to create database tables of the User Manager. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        UserRealm realm = null;
        try {
            realm = new DefaultRealm();
            DefaultRealmConfig config = (DefaultRealmConfig) realm.getRealmConfiguration();
            config.setConnectionURL("jdbc:hsqldb:mem:aname");
            realm.init(config);
        } catch (UserStoreException e) {
            String msg = "Failed to initialize the user manager. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        this.userRealm = new RegistryRealm(realm);
    }

    public Object getRealmConfiguration() throws UserStoreException {
        return userRealm.getRealmConfiguration();
    }

    public void init(Object o) throws UserStoreException {
        userRealm.init(o);
    }

    public Authenticator getAuthenticator() throws UserStoreException {
        return userRealm.getAuthenticator();
    }

    public Authorizer getAuthorizer() throws UserStoreException {
        return userRealm.getAuthorizer();
    }

    public UserStoreReader getUserStoreReader() throws UserStoreException {
        return userRealm.getUserStoreReader();
    }

    public UserStoreAdmin getUserStoreAdmin() throws UserStoreException {
        return userRealm.getUserStoreAdmin();
    }

    public AccessControlAdmin getAccessControlAdmin() throws UserStoreException {
        return userRealm.getAccessControlAdmin();
    }
    
    public void cleanUp() throws UserStoreException{
	
    }
}
