/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.ui;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.Resource;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.core.RegistryResources;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

/**
 *
 */
public class CarbonConnection extends URLConnection {

    private static final Log log = LogFactory.getLog(CarbonConnection.class);
    
    private String servicePath = "services";

    private String contextRoot = "";

    private String httpPort = "9763";

    private String httpsPort = "9443";

    private byte[] buf = null;

    /**
     * Constructs a URL connection to the specified URL. A connection to
     * the object referenced by the URL is not created.
     *
     * @param url     the specified URL.
     * @param context BundleContext
     */
    protected CarbonConnection(URL url, BundleContext context) {
        super(url);
        ServiceReference reference = context.getServiceReference(RegistryService.class.getName());
        if (reference != null && buf == null) {
            RegistryService registryService = (RegistryService) context.getService(reference);
            try {
                Registry systemRegistry = registryService.getSystemRegistry();
                Resource resource = systemRegistry.get(RegistryResources.CONNECTION_PROPS);
                String servicePath = resource.getProperty("service-path");
                String contextRoot = resource.getProperty("context-root");
                contextRoot = contextRoot.equals("/") ? "" : contextRoot;
                this.servicePath = servicePath;
                this.contextRoot = contextRoot;
                this.httpPort = System.getProperty("carbon.http.port", "9763");
                this.httpsPort = System.getProperty("carbon.https.port", "9443");
                buf = ("var SERVICE_PATH=\"" + servicePath + "\";\n" +
                        "var ROOT_CONTEXT=\"" + contextRoot + "\";\n" +
                        "var HTTP_PORT=" + httpPort + ";\n" +
                        "var HTTPS_PORT=" + httpsPort + ";\n").getBytes();
                resource.discard();
            } catch (Exception e) {
                String msg = "Error occurred while getting connection properties";
                log.error(msg, e);
            }
        }
    }

    public void connect() throws IOException {
        //Ignore; no need to have this
    }

    public InputStream getInputStream() throws IOException {
//        String s = getURL().getPath();
        return new ByteArrayInputStream(buf);
    }

    public String getContentType() {
        return "text/javascript";
    }

    public int getContentLength() {
        return buf.length;
    }


}