package org.wso2.carbon.ui;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.Bundle;
import org.osgi.framework.ServiceReference;
import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.ui.deployment.beans.CarbonUIDefinitions;

public class CarbonSecuredHttpContext extends SecuredComponentEntryHttpContext {

    public static String LOGGED_USER = "logged-user";
    private static Log log = LogFactory.getLog(CarbonSecuredHttpContext.class);
    private Bundle bundle = null;

    public CarbonSecuredHttpContext(Bundle bundle, String s, Registry registry) {
        super(bundle, s);
        this.registry = registry;
        this.bundle = bundle;
    }

    public boolean handleSecurity(HttpServletRequest request,
                                  HttpServletResponse response) throws IOException {
        //TODO : remove hard coded "carbon" text
        String requestedURI = request.getRequestURI();
        HttpSession session;
        String sessionId;
        boolean authenticated = false;
        try {
            session = request.getSession();
            sessionId = session.getId();
            Boolean authenticatedObj = (Boolean) session.getAttribute("authenticated");            
            if (authenticatedObj != null) {
                authenticated = authenticatedObj.booleanValue();
            }            
        } catch (Exception e) {
            return false;
        }

        //we no longer support / as the uri to carbon
        if(requestedURI.equals("/")){
        	PrintWriter pw = response.getWriter();
        	pw.println("<html>");
        	pw.println("<body>");
        	pw.println("<b>Available contexts</b>");
        	pw.println("<li><a href=\"./carbon\">carbon</a></li>");
        	pw.println("</body>");
        	pw.println("</html>");
        	pw.flush();
        	return false;
        }
        //when war is deployed on top of an existing app server we cannot use root context
        //for deployment. Hence a new context is added.
        //Now url changes from eg: carbon/admin/index.jsp to wso2/carbon/admin/index.jsp
        //In this case before doing anything, we need to remove web app context (eg: wso2) .       
        String tmp = requestedURI;
        String customWarContext = "";
        if(requestedURI.startsWith("/carbon") 
        		&& !(requestedURI.startsWith("/carbon/carbon/"))){
        	//one can name the folder as 'carbon'
        	requestedURI = tmp;
        }else if(requestedURI.indexOf("filedownload") == -1 
        		&& requestedURI.indexOf("fileupload") == -1){
        	//replace first context
        	String tmp1 = tmp.replaceFirst("/", "");
        	int end = tmp1.indexOf("/");
        	if(end > -1){
            	customWarContext = tmp1.substring(0,end);
            	if(customWarContext.equals("registry")){
            		requestedURI = tmp;
            	}else{
            		requestedURI = tmp.substring(end+1);
            	}
            }
        }
        
        //Disabling http access to admin console
        //user guide documents should be allowed to access via http protocol
        if (!request.isSecure()
                && !(requestedURI.endsWith(".html"))){

        	//by passing items required for try-it & IDE plugins
        	if(requestedURI.endsWith(".css") ||
                requestedURI.endsWith(".gif") ||
                requestedURI.endsWith(".GIF") ||                
                requestedURI.endsWith(".jpg") ||
                requestedURI.endsWith(".JPG") ||                
                requestedURI.endsWith(".png") ||
                requestedURI.endsWith(".PNG") ||                
                requestedURI.endsWith(".xsl") ||
                requestedURI.endsWith(".xslt") ||
                requestedURI.endsWith(".js") ||
                requestedURI.endsWith(".ico") ||
                requestedURI.endsWith("/filedownload") ||
                requestedURI.endsWith("/fileupload") ||
                requestedURI.indexOf("/fileupload/") > -1 ||
                requestedURI.indexOf("admin/jsp/WSRequestXSSproxy_ajaxprocessor.jsp") > -1 ||
                requestedURI.indexOf("registry/atom") > -1 ||
                requestedURI.indexOf("registry/resource") > -1){
        		return true;
        	}
        	
            ServletContext servletContext = session.getServletContext();
            String adminConsoleURL = CarbonUIUtil.getAdminConsoleURL(request);
            if (adminConsoleURL != null) {
                if (log.isTraceEnabled()) {
                    log.trace("Request came to admin console via http.Forwarding to : " + adminConsoleURL);
                }
                response.sendRedirect(adminConsoleURL);
                return false;
            }
        }
        
        //remove web application context & 'carbon' sub context from uri
        String resourceURI = requestedURI.replaceFirst("/carbon/", "../");

        if (log.isDebugEnabled()) {
            log.debug("CarbonSecuredHttpContext -> handleSecurity() requestURI:" + requestedURI + " id:" + sessionId + " resourceURI:" + resourceURI);
        }

        //retrieve urls that should be by-passed from security check
        HashMap<String, String> urlsToBeByPassed = new HashMap<String, String>();
        if (bundle != null) {
            ServiceReference reference = bundle.getBundleContext().getServiceReference(CarbonUIDefinitions.class.getName());
            CarbonUIDefinitions carbonUIDefinitions = null;
            if (reference != null) {
                carbonUIDefinitions = (CarbonUIDefinitions) bundle.getBundleContext().getService(reference);
                if (carbonUIDefinitions != null) {
                    urlsToBeByPassed = carbonUIDefinitions.getUnauthenticatedUrls();
                    /*
                         if(log.isDebugEnabled()){
                             Iterator<String> itr = urlsToBeByPassed.keySet().iterator();
                             while(itr.hasNext()){
                                 log.debug("Should bypass url : "+itr.next());
                             }
                         }
                         */
                }
            }
        }

        //if the current uri is marked to be by-passed, let it pass through
        if (!urlsToBeByPassed.isEmpty()) {
            if (urlsToBeByPassed.containsKey(resourceURI)) {
                if (log.isDebugEnabled()) {
                    log.debug("By passing authentication check for URI : " + resourceURI);
                }
                return true;
            }
        }

        if (requestedURI.indexOf("login.jsp") > -1
                || requestedURI.indexOf("admin/layout/template.jsp") > -1
                || requestedURI.endsWith("/filedownload")
                || requestedURI.endsWith("/fileupload")
                || requestedURI.indexOf("/fileupload/") > -1                 
                || requestedURI.indexOf("admin/jsp/WSRequestXSSproxy_ajaxprocessor.jsp") > -1) {

            if (requestedURI.indexOf("login.jsp") > -1 && authenticated) {
                //User has typed the login page url, while being logged in
                response.sendRedirect("/carbon/admin/index.jsp");
                return true;
            } else {
                return true;
            }
        }
        boolean sessionAuth = false;

		if (requestedURI.indexOf("login_action.jsp") > -1) {
			String userName = request.getParameter("username");
			String password = request.getParameter("password");
			String ssoSessionId = request.getParameter("ssoSessionId");

			try {

				if (ssoSessionId != null && password == null) {
					// This is SSO related login - we don't have the password.
					// Now the back-end service need to validate ssoSession id with back-end
					// SSO-Service.
					HttpSession currentSession = request.getSession(false);
					String loggedinUser = (String)session.getAttribute(LOGGED_USER);
					if (currentSession != null && loggedinUser != null) {
						currentSession.removeAttribute(MenuAdminClient.USER_MENU_ITEMS);
					}
					sessionAuth = authenticate(userName, ssoSessionId, request);
				} else {
					sessionAuth = authenticate(userName, password, request);
				}

                if (sessionAuth) {
                    session.setAttribute("authenticated", sessionAuth);

                    //TODO: I'm commenting out code which redirects to the previously requested path
                    // this bit seems to be buggy, often returning error pages.
                    /*String previousRequestUri = (String) session.getAttribute("requestedUri");
                    if (previousRequestUri != null && previousRequestUri.trim().length() > 0) {
                        if (log.isTraceEnabled()) {
                            log.trace("Forwarding to previously request page : " + previousRequestUri);
                        }
                        response.sendRedirect(previousRequestUri);
                    } else {
                        response.sendRedirect("../../carbon/admin/index.jsp?loginStatus=true");
                    }*/

                    response.sendRedirect("../../carbon/admin/index.jsp?loginStatus=true");
                    session.setAttribute(LOGGED_USER, userName);
                } else {
                    response.sendRedirect("../../carbon/admin/login.jsp?loginStatus=false");
                }

            } catch (Exception e) {
                log.error("error occurred while login", e);
                response.sendRedirect("../../carbon/admin/login.jsp?loginStatus=failed");
            }
            return false;
        } else if (requestedURI.indexOf("logout_action.jsp") > 1) {
            unauthenticate(request); // Logout the user from the backend
            HttpSession currentSession = request.getSession(false);
            if (currentSession != null) {
                //check if current session has expired
                session.removeAttribute(LOGGED_USER);
                try {
                    session.invalidate();
                } catch (Exception ignored) { // Ignore exception when invalidating and invalidated session
                }
            }
            response.sendRedirect("../../carbon/admin/index.jsp");
            return false;
        }
        if (requestedURI.endsWith("/carbon/")) {
            response.sendRedirect("./admin/index.jsp");
            return false;
        } 
        else if (requestedURI.indexOf("/registry/atom") == -1 && requestedURI.endsWith("/carbon")) {
        	String redirectUrl = "/carbon/admin/index.jsp";
        	if(customWarContext != "" && customWarContext.trim().length() > 0){
        		redirectUrl = "/"+customWarContext+redirectUrl;
        	}
            response.sendRedirect(redirectUrl);
            return false;
        }else if (requestedURI.endsWith(".css") ||
                requestedURI.endsWith(".gif") ||
                requestedURI.endsWith(".GIF") ||                
                requestedURI.endsWith(".jpg") ||
                requestedURI.endsWith(".JPG") ||                
                requestedURI.endsWith(".png") ||
                requestedURI.endsWith(".PNG") ||                
                requestedURI.endsWith(".xsl") ||
                requestedURI.endsWith(".xslt") ||
                requestedURI.endsWith(".js") ||
                requestedURI.startsWith("/registry") ||
                requestedURI.endsWith(".html") ||
                requestedURI.endsWith(".ico")) {
            return true;
        } else if (requestedURI.endsWith(".jsp") && authenticated) {
            return true;
        }

        if (!authenticated) {
            if (requestedURI.endsWith("ajaxprocessor.jsp")) {  //To prevent login page appearing when ajax calls happen
                return true;
            } else {
                //saving originally requested url
                //should not forward to error page after login
                if (!requestedURI.endsWith("admin/error.jsp")) {
                    String queryString = request.getQueryString();
                    String tmpURI = "";
                    if (queryString != null) {
                    	tmpURI = requestedURI + "?" + queryString;
                    } else {
                    	tmpURI = requestedURI;
                    }
                    tmpURI = "../.."+tmpURI;
                    request.getSession(false).setAttribute("requestedUri", tmpURI);
                }
                response.sendRedirect("../admin/login.jsp");
            }
            return false;
        }

        if (request.getSession().isNew()) {
            response.sendRedirect("../admin/login.jsp");
            return false;
        }

        return true;
    }
}
