package org.wso2.carbon.ui;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletConfig;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.ui.deployment.beans.CarbonUIDefinitions;
import org.wso2.carbon.ui.deployment.beans.Menu;
import org.wso2.carbon.utils.ServerConfiguration;
import org.wso2.carbon.utils.NetworkUtils;
import org.wso2.carbon.utils.ServerConstants;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Locale;

/**
 * Utility class for Carbon UI
 */
public class CarbonUIUtil {
    private static Log log = LogFactory.getLog(CarbonUIUtil.class);
    private static BundleContext bundleContext = null;

    public static void setBundleContext(BundleContext context) {
        bundleContext = context;
    }

    public static BundleContext getBundleContext() {
        return bundleContext;
    }

    public static String getServerURL(ServletContext servletContext, HttpSession httpSession) {
        Object obj = httpSession.getAttribute(CarbonConstants.SERVER_URL);
        if (obj != null && obj instanceof String) {
            //Server URL is present in the servlet session
            return (String) obj;
        }
        return (String) servletContext.getAttribute(CarbonConstants.SERVER_URL);
    }

    /**
     * Returns url to admin console.
     * eg: https://192.168.1.201:9443/wso2/carbon
     *
     * @param request
     * @return
     */
    public static String getAdminConsoleURL(HttpServletRequest request) {
        String requestURI = request.getRequestURI();

        int indexOfCarbon = requestURI.indexOf("/carbon");
        String context = "/";
        if(indexOfCarbon != -1 && indexOfCarbon != 0){
            requestURI.substring(0, indexOfCarbon - 1);
        }

        String hostName = "localhost";
        try {
            hostName = NetworkUtils.getLocalHostname();
        } catch (Exception ignored) {
        }
        String url = "https://" + hostName + ":" +
                     System.getProperty(ServerConstants.HTTPS_PORT) +
                     context + "/carbon/";
        return url;
    }

    /**
     * Get a ServerConfiguration Property
     * @param propertyName  Name of the property
     * @return  the property
     */
    public static String getServerConfigurationProperty(String propertyName){
        String value = null;
         ServerConfiguration serverConfig = null;
        ServiceTracker serverConfigTracker = new ServiceTracker(bundleContext, ServerConfiguration.class.getName(), null);
        serverConfigTracker.open();
        if (serverConfigTracker.getService() != null) {
            serverConfig = (ServerConfiguration) serverConfigTracker.getService();
            value = serverConfig.getFirstProperty(propertyName);
        }
        return value;
    }

    public static boolean isContextRegistered(ServletConfig config, String context) {
        URL url;
        try {
            url = config.getServletContext().getResource(context);
        } catch (MalformedURLException e) {
            return false;
        }
        if (url == null) {
            return false;
        } else if (url.toString().indexOf(context) != -1) {
            return true;
        }
        return false;
    }

    /**
     * Returns internationalized string for supplied key.
     *
     * @param key        - key to look for
     * @param i18nBundle - resource bundle
     * @param language   - language
     * @return internationalized key
     *         value of key, if no value can be derived
     */
    public static String geti18nString(String key, String i18nBundle, String language) {
        Locale locale = new Locale(language);
        String text = geti18nString(key, i18nBundle, locale);
        return text;
    }

    /**
     * Returns internationalized string for supplied key.
     *
     * @param key        - key to look for
     * @param i18nBundle - resource bundle
     * @param Locale     - locale
     * @return internationalized key
     *         value of key, if no value can be derived
     */
    public static String geti18nString(String key, String i18nBundle, Locale locale) {
        java.util.ResourceBundle resourceBundle = null;
        try {
            resourceBundle = java.util.ResourceBundle.getBundle(i18nBundle, locale);
        } catch (java.util.MissingResourceException e) {
            if (log.isDebugEnabled()) {
                log.debug("Cannot find resource bundle : " + i18nBundle
                          + " for locale : " + locale);
            }
        }

        String text = key;
        if (resourceBundle != null) {
            String tmp = null;
            try {
                tmp = resourceBundle.getString(key);
            } catch (java.util.MissingResourceException e) {
                //Missing key should not be a blocking factor for UI rendering
                if (log.isDebugEnabled()) {
                    log.debug("Cannot find resource for key :" + key);
                }
            }
            if (tmp != null) {
                text = tmp;
            }
        }
        return text;
    }

    /**
     * Removed menu item from current user's session.
     * Only current user's menu items are effected.
     *
     * @param menuId
     * @param request
     * @see CarbonUIDefinitions#removeMenuDefinition(String)
     */
    public static void removeMenuDefinition(String menuId, HttpServletRequest request) {
        //TODO : consider removing child menu items as well
        ArrayList<Menu> modifiedMenuDefs = new ArrayList<Menu>();
        Menu[] currentMenus = (Menu[]) request.getSession().getAttribute(MenuAdminClient.USER_MENU_ITEMS);
        boolean modified = false;
        if (currentMenus != null) {
            if (menuId != null && menuId.trim().length() > 0) {
                for (int a = 0; a < currentMenus.length; a++) {
                    Menu menu = currentMenus[a];
                    if (menu != null) {
                        if (!menuId.equals(menu.getId())) {
                            modifiedMenuDefs.add(menu);
                            modified = true;
                        } else {
                            if (log.isDebugEnabled()) {
                                log.debug("Removing menu item : " + menuId);
                            }
                        }
                    }
                }
                if (modified) {
                    Menu[] newMenuDefs = new Menu[modifiedMenuDefs.size()];
                    newMenuDefs = modifiedMenuDefs.toArray(newMenuDefs);
                    request.getSession().setAttribute(MenuAdminClient.USER_MENU_ITEMS, newMenuDefs);
				}
			}
		}
	}
}