/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.ui.internal;

import static org.wso2.carbon.CarbonConstants.WSO2CARBON_NS;
import static org.wso2.carbon.CarbonConstants.PRODUCT_XML;
import static org.wso2.carbon.CarbonConstants.PRODUCT_XML_PROPERTY;
import static org.wso2.carbon.CarbonConstants.PRODUCT_XML_PROPERTIES;
import static org.wso2.carbon.CarbonConstants.PRODUCT_XML_WSO2CARBON;

import java.io.IOException;
import java.io.InputStream;
import java.net.ContentHandler;
import java.net.URL;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import javax.servlet.Servlet;
import javax.servlet.ServletContext;
import javax.xml.namespace.QName;
import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.equinox.http.helper.ContextPathServletAdaptor;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.http.HttpContext;
import org.osgi.service.http.HttpService;
import org.osgi.service.packageadmin.PackageAdmin;
import org.osgi.service.url.URLConstants;
import org.osgi.service.url.URLStreamHandlerService;
import org.osgi.util.tracker.ServiceTracker;
import org.osgi.util.tracker.ServiceTrackerCustomizer;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.ui.CarbonProtocol;
import org.wso2.carbon.ui.CarbonSecuredHttpContext;
import org.wso2.carbon.ui.CarbonUIUtil;
import org.wso2.carbon.ui.TextJavascriptHandler;
import org.wso2.carbon.ui.TilesJspServlet;
import org.wso2.carbon.ui.Utils;
import org.wso2.carbon.ui.deployment.UIBundleDeployer;
import org.wso2.carbon.ui.deployment.beans.CarbonUIDefinitions;
import org.wso2.carbon.ui.deployment.beans.CustomUIDefenitions;
import org.wso2.carbon.ui.transports.FileDownloadServlet;
import org.wso2.carbon.ui.transports.FileUploadServlet;
import org.wso2.carbon.utils.ConfigurationContextService;
import org.wso2.carbon.utils.ServerConfiguration;

public class Activator implements BundleActivator, ServiceTrackerCustomizer {
    private static Log log = LogFactory.getLog(Activator.class);

    private ServiceTracker packageAdminTracker;

    private ServiceTracker configContextTracker;

    private ServiceTracker serverConfigTracker;

    private static PackageAdmin packageAdmin;

    private BundleContext bundleContext;

    private Servlet adaptedJspServlet;


    public void start(BundleContext context) throws Exception {
        this.bundleContext = context;
        packageAdminTracker = new ServiceTracker(context, PackageAdmin.class.getName(), this);
        packageAdminTracker.open();

        configContextTracker = new ServiceTracker(bundleContext, ConfigurationContextService.class.getName(), this);
        configContextTracker.open();

        serverConfigTracker = new ServiceTracker(bundleContext, ServerConfiguration.class.getName(), this);
        serverConfigTracker.open();

        try {
            ConfigurationContext configContext;
            if (configContextTracker.getService() != null) {
                configContext = ((ConfigurationContextService)
                        configContextTracker.getService()).getClientConfigContext();
            } else {
                throw new Exception("ConfigurationContext Service is not available");
            }

            ServerConfiguration serverConfig;
            if (serverConfigTracker.getService() != null) {
                serverConfig = (ServerConfiguration) serverConfigTracker.getService();
            } else {
                throw new Exception("ServerConfiguration Service is not available");
            }

            CarbonUIDefinitions carbonUIDefinitions = new CarbonUIDefinitions();
            context.registerService(CarbonUIDefinitions.class.getName(), carbonUIDefinitions, null);

            // create a CustomUIDefinitions object and set it as a osgi service. UIBundleDeployer can access this
            // service and populate it with custom UI definitions of the deployed UI bundle, if available.
            CustomUIDefenitions customUIDefenitions = new CustomUIDefenitions();
            context.registerService(CustomUIDefenitions.class.getName(), customUIDefenitions, null);

            Hashtable<String, String[]> properties1 = new Hashtable<String, String[]>();
            properties1.put(URLConstants.URL_HANDLER_PROTOCOL, new String[]{"carbon"});
            context.registerService(URLStreamHandlerService.class.getName(), new CarbonProtocol(context), properties1);

            Hashtable<String, String[]> properties3 = new Hashtable<String, String[]>();
            properties3.put(URLConstants.URL_CONTENT_MIMETYPE, new String[]{"text/javascript"});
            context.registerService(ContentHandler.class.getName(), new TextJavascriptHandler(), properties3);

            ServiceReference reference = context.getServiceReference(HttpService.class.getName());
            if (reference == null) {
                throw new Exception("An instance of HttpService is not available");
            }
            final HttpService httpService = (HttpService) context.getService(reference);

            Dictionary<String, String> initparams = new Hashtable<String, String>();
            initparams.put("servlet-name", "TilesServlet");
            initparams.put("definitions-config", "/WEB-INF/tiles/main_defs.xml");
            initparams.put("org.apache.tiles.context.TilesContextFactory",
                    "org.apache.tiles.context.enhanced.EnhancedContextFactory");
            initparams.put("org.apache.tiles.factory.TilesContainerFactory.MUTABLE", "true");
            initparams.put("org.apache.tiles.definition.DefinitionsFactory",
                    "org.wso2.carbon.tiles.CarbonUrlDefinitionsFactory");

            String webContext = serverConfig.getFirstProperty(CarbonConstants.WEB_CONTEXT);
            if (webContext == null){
                webContext = Utils.getWebContextName();            	
            }

            String serverURL = serverConfig.getFirstProperty(CarbonConstants.SERVER_URL);

            ServiceReference regRef = context.getServiceReference(RegistryService.class.getName());
            if (regRef == null) {
                throw new Exception("RegistryService is not found");
            }
            RegistryService registryService = (RegistryService) context.getService(regRef);
            Registry registry = registryService.getSystemRegistry();

            HttpContext commonContext = new CarbonSecuredHttpContext(context.getBundle(), "/web", registry);

            httpService.registerServlet("/", new org.apache.tiles.web.startup.TilesServlet(), initparams,
                    commonContext);
            httpService.registerResources("/" + webContext, "/", commonContext);

            adaptedJspServlet = new ContextPathServletAdaptor(
                    new TilesJspServlet(context.getBundle(), "/web"), "/" + webContext);
            httpService.registerServlet("/" + webContext + "/*.jsp", adaptedJspServlet, null, commonContext);

            ServletContext jspServletContext = adaptedJspServlet.getServletConfig().getServletContext();
            jspServletContext.setAttribute("registry", registryService);
            jspServletContext.setAttribute(CarbonConstants.CONFIGURATION_CONTEXT, configContext);
            jspServletContext.setAttribute(CarbonConstants.BUNDLE_CLASS_LOADER, this.getClass().getClassLoader());
            jspServletContext.setAttribute(CarbonConstants.SERVER_URL, serverURL);

            // set the CustomUIDefinitions object as an attribute of servlet context, so that the Registry UI bundle
            // can access the custom UI details within JSPs.
            jspServletContext.setAttribute(CustomUIDefenitions.CUSTOM_UI_DEFENITIONS, customUIDefenitions);

            //Registering filedownload servlet
            Servlet fileDownloadServlet = new ContextPathServletAdaptor(new FileDownloadServlet(
                    context, (ConfigurationContextService)configContextTracker.getService()), "/filedownload");
            httpService.registerServlet("/filedownload", fileDownloadServlet, null, commonContext);
            fileDownloadServlet.getServletConfig().getServletContext().setAttribute(
                    CarbonConstants.SERVER_URL, serverURL);

            //Registering fileupload servlet
            Servlet fileUploadServlet = new ContextPathServletAdaptor(new FileUploadServlet(
                    context, configContext, webContext), "/fileupload");
            httpService.registerServlet("/fileupload", fileUploadServlet, null, commonContext);
            fileUploadServlet.getServletConfig().getServletContext().setAttribute(
                    CarbonConstants.SERVER_URL, serverURL);

            //saving bundle context for future reference within CarbonUI Generation
            CarbonUIUtil.setBundleContext(context);

            //UIBundleDeployer.setBundleContext(context);
            context.addBundleListener(new UIBundleDeployer(bundleContext, commonContext));

            if (log.isDebugEnabled()) {
                log.debug("Starting web console using context : " + webContext);
            }
            
            //read product.xml
            readProductXML(jspServletContext);            
        } catch (Exception e) {
            String msg = "Failed to start Carbon UI";
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    /**
     * reads product.xml contained within styles bundle of a product.
     * product.xml contains properties like userforum, mailing list,etc.. which are
     * specific to the product.
     * @param jspServletContext
     * @throws IOException
     * @throws XMLStreamException
     * @throws FactoryConfigurationError
     */
	private void readProductXML(ServletContext jspServletContext)
			throws IOException, XMLStreamException, FactoryConfigurationError {
		Enumeration<URL> e= bundleContext.getBundle().findEntries("META-INF", PRODUCT_XML, true);
		if(e != null){
			URL url = (URL)e.nextElement();
		    InputStream inputStream = url.openStream();            	
		    XMLStreamReader streamReader =
		        XMLInputFactory.newInstance().createXMLStreamReader(inputStream);
		    StAXOMBuilder builder = new StAXOMBuilder(streamReader);
		    OMElement document = builder.getDocumentElement();
		    OMElement propsEle =
		        document.getFirstChildWithName(new QName(WSO2CARBON_NS, PRODUCT_XML_PROPERTIES));
		    if(propsEle != null){
		    	Iterator<OMElement> properties = propsEle.getChildrenWithName(new QName(WSO2CARBON_NS, PRODUCT_XML_PROPERTY));
		    	while(properties.hasNext()){
		    		OMElement property = properties.next();
		    		String propertyName = property.getAttributeValue(new QName("name"));
		    		String value = property.getText();
		    		if(log.isDebugEnabled()){
		    			log.debug(PRODUCT_XML+": "+propertyName+"="+value);
		    		}
		    		jspServletContext.setAttribute(PRODUCT_XML_WSO2CARBON + propertyName, value);              		
		    	}                	
 		    }
		}
	}

    public void stop(BundleContext bundleContext) throws Exception {
        //TODO release all the trackers opened in the UIBundleListener instance
    }


    public static synchronized Bundle getBundle(Class clazz) {
        if (packageAdmin == null)
            throw new IllegalStateException("Not started"); //$NON-NLS-1$
        return packageAdmin.getBundle(clazz);
    }

    public static Bundle[] getFragments(Bundle bundle) {
        if (packageAdmin == null)
            throw new IllegalStateException("Not started"); //$NON-NLS-1$
        return packageAdmin.getFragments(bundle);
    }


    public Object addingService(ServiceReference reference) {
        Object serviceObject;
        synchronized (Activator.class) {

            serviceObject = bundleContext.getService(reference);
            if (serviceObject instanceof PackageAdmin) {
                packageAdmin = (PackageAdmin) serviceObject;

//            } else if(serviceObject instanceof ConfigurationContextService){
//                ConfigurationContext configContext = ((ConfigurationContextService)
//                        configContextTracker.getService()).getClientConfigContext();
//                adaptedJspServlet.getServletConfig().getServletContext().setAttribute(CarbonConstants.CONFIGURATION_CONTEX,
//                configContext);
            }
        }
        return serviceObject;
    }

    public void modifiedService(ServiceReference reference, Object service) {
    }

    public void removedService(ServiceReference reference, Object service) {
        synchronized (Activator.class) {
            bundleContext.ungetService(reference);
            packageAdmin = null;
        }
    }


}