/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.user.core.dbcreators;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;

public class DerbyDatabaseCreator implements DatabaseCreator {
    // TODO Start -delete
    public static String USER_TABLE = "CREATE TABLE UM_USERS ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY," + "USER_NAME VARCHAR(255) NOT NULL,"
            + "USER_PASSWORD VARCHAR(255) NOT NULL, " + "PRIMARY KEY (ID), " + "UNIQUE(USER_NAME))";

    public static String USER_ATTR_TABLE = "CREATE TABLE UM_USER_ATTRIBUTES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY," + "ATTR_NAME VARCHAR(255) NOT NULL,"
            + "ATTR_VALUE VARCHAR(255), " + "USER_ID INTEGER, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static String ROLE_TABLE = "CREATE TABLE UM_ROLES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ROLE_NAME VARCHAR(255) NOT NULL, "
            + "PRIMARY KEY (ID), " + "UNIQUE(ROLE_NAME))";

    public static String ROLE_ATTR_TABLE = "CREATE TABLE UM_ROLE_ATTRIBUTES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ATTR_NAME VARCHAR(255) NOT NULL, "
            + "ATTR_VALUE VARCHAR(255), " + "ROLE_ID INTEGER, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static String PERMISSION_TABLE = "CREATE TABLE UM_PERMISSIONS ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "RESOURCE_ID VARCHAR(255), "
            + "ACTION VARCHAR(255) NOT NULL, " + "PRIMARY KEY (ID))";

    public static String ROLE_PERMISSION_TABLE = "CREATE TABLE UM_ROLE_PERMISSIONS ( "
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "PERMISSION_ID INTEGER NOT NULL, "
            + "ROLE_ID INTEGER NOT NULL, " + "IS_ALLOWED SMALLINT NOT NULL, "
            + "UNIQUE (PERMISSION_ID, ROLE_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE  CASCADE, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static String USER_PERMISSION_TABLE = "CREATE TABLE UM_USER_PERMISSIONS ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "PERMISSION_ID INTEGER NOT NULL, "
            + "USER_ID INTEGER NOT NULL, " + "IS_ALLOWED SMALLINT NOT NULL, "
            + "UNIQUE (PERMISSION_ID, USER_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static String USER_ROLES_TABLE = "CREATE TABLE UM_USER_ROLES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ROLE_ID INTEGER NOT NULL, "
            + "USER_ID INTEGER NOT NULL, " + "UNIQUE (USER_ID, ROLE_ID), "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    // ////////////////////////////////////////////////////////////////
    // Hybrid sql statements
    // ////////////////////////////////////////////////////////////////
    
    public static String HYBRID_ROLE_TABLE = "CREATE TABLE HYBRID_ROLES ("
        + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ROLE_ID VARCHAR(255) NOT NULL, "
        + "PRIMARY KEY (ID), " + "UNIQUE(ROLE_ID))";
    
    public static final String HYBRID_USER_ROLE_TABLE = "CREATE TABLE HYBRID_USER_ROLES ("
        + "ID INTEGER GENERATED ALWAYS AS IDENTITY," + "USER_ID VARCHAR(255),"
        + "ROLE_ID VARCHAR(255) NOT NULL," + "PRIMARY KEY (ID))";
    
    public static final String HYBRID_PERMISSION_TABLE = "CREATE TABLE HYBRID_PERMISSIONS ("
            + "ID INTEGER GENERATED ALWAYS AS IDENTITY," + "RESOURCE_ID VARCHAR(255),"
            + "ACTION VARCHAR(255) NOT NULL," + "PRIMARY KEY (ID))";

    public static final String HYBRID_ROLE_PERMISSION_TABLE = "CREATE TABLE HYBRID_ROLE_PERMISSIONS ("
            + "ID INTEGER GENERATED ALWAYS AS IDENTITY," + "PERMISSION_ID INTEGER NOT NULL,"
            + "ROLE_ID VARCHAR(255) NOT NULL," + "IS_ALLOWED SMALLINT NOT NULL,"
            + "UNIQUE (PERMISSION_ID, ROLE_ID),"
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES HYBRID_PERMISSIONS(ID) ON DELETE  CASCADE,"
            + "PRIMARY KEY (ID))";

    public static final String HYBRID_USER_PERMISSION_TABLE = "CREATE TABLE HYBRID_USER_PERMISSIONS ("
            + "ID INTEGER GENERATED ALWAYS AS IDENTITY," + "PERMISSION_ID INTEGER NOT NULL,"
            + "USER_ID VARCHAR(255) NOT NULL," + "IS_ALLOWED SMALLINT NOT NULL,"
            + "UNIQUE (PERMISSION_ID, USER_ID),"
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES HYBRID_PERMISSIONS(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    private DataSource dataSource = null;

    public DerbyDatabaseCreator(DataSource ds) {
        this.dataSource = ds;
    }

    public void createDefaultDatabaseTables() throws SQLException {
        Connection dbConnection = dataSource.getConnection();
        DatabaseMetaData dbmd = dbConnection.getMetaData();
        ResultSet rs = dbmd.getTables(null, null, "UM_USERS", null);
        if (rs.next() == false) {
            Statement s = dbConnection.createStatement();
            s.executeUpdate(USER_TABLE);
            s.executeUpdate(USER_ATTR_TABLE);
            s.executeUpdate(ROLE_TABLE);
            s.executeUpdate(ROLE_ATTR_TABLE);
            s.executeUpdate(PERMISSION_TABLE);
            s.executeUpdate(ROLE_PERMISSION_TABLE);
            s.executeUpdate(USER_PERMISSION_TABLE);
            s.executeUpdate(USER_ROLES_TABLE);
            dbConnection.commit();
        }
        dbConnection.close();
    }

    public void createHybridDatabaseTables() throws SQLException {
        Connection dbConnection = dataSource.getConnection();
        DatabaseMetaData dbmd = dbConnection.getMetaData();
        ResultSet rs = dbmd.getTables(null, null, "HYBRID_USER_ROLES", null);
        if (rs.next() == false) {
            Statement s = dbConnection.createStatement();
            s.executeUpdate(HYBRID_ROLE_TABLE);
            s.executeUpdate(HYBRID_USER_ROLE_TABLE);
            s.executeUpdate(HYBRID_PERMISSION_TABLE);
            s.executeUpdate(HYBRID_ROLE_PERMISSION_TABLE);
            s.executeUpdate(HYBRID_USER_PERMISSION_TABLE);
            dbConnection.commit();
        }
        dbConnection.close();
    }
}
