package org.wso2.carbon.user.core.hybrid;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.def.DefaultAccessControlAdmin;
import org.wso2.carbon.user.core.def.DefaultAuthorizer;
import org.wso2.carbon.user.core.def.DefaultRealmConstants;
import org.wso2.carbon.user.core.def.DefaultUserStoreAdmin;
import org.wso2.carbon.user.core.def.SQLHelper;
import org.wso2.carbon.user.core.def.util.DefaultCommons;
import org.wso2.carbon.user.core.i18n.Messages;

public class HybridInternalRoleAdmin{
     
    private DefaultUserStoreAdmin defUSAdmin = null;
    protected SQLHelper sqlHelper = null;
    protected DataSource dataSource = null;
    
    
    public HybridInternalRoleAdmin(DataSource dataSource){
        this.dataSource = dataSource;
        sqlHelper = new HybridSQLHelper();
        defUSAdmin = new DefaultUserStoreAdmin(dataSource, sqlHelper);
    }
    
    
    public String[] getInternalRolesOfUser(String userName) throws UserStoreException{
       return defUSAdmin.getUserRoles(userName);
    }
    
    public String[] getUsersInInternalRole(String roleName) throws UserStoreException{
        return defUSAdmin.getUsersInRole(roleName);
    }
    
    public void addInternalRole(String roleName) throws UserStoreException {
        defUSAdmin.addRole(roleName);
    }
    
    public String[] getInternalRoles()  throws UserStoreException {
        return defUSAdmin.getAllRoleNames();
    }
    
    public void addUserToInternalRole(String userName, String roleName) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement addUserToRoleStmt = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
        
            addUserToRoleStmt = dbConnection
                    .prepareStatement(sqlHelper.getSQL(DefaultRealmConstants.ADD_USER_ROLE));
            addUserToRoleStmt.setString(1, userName);
            addUserToRoleStmt.setString(2, roleName);
            addUserToRoleStmt.executeUpdate();
            dbConnection.commit();
            addUserToRoleStmt.close();
        } catch (SQLException e) {
            e.printStackTrace();
            throw new UserStoreException(Messages.getMessage("errorAddingUserToRole"), e);
        } finally {
            try {
                if (addUserToRoleStmt!=null) {
                    addUserToRoleStmt.close();
                }
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        } 
    }

    public void removeUserFromInternalRole(String userName, String roleName) throws UserStoreException {
        defUSAdmin.removeUserFromRole(userName, roleName);
    }
    
    public void deleteInternalRole(String roleName) throws UserStoreException {
        defUSAdmin.deleteRole(roleName);        
    }   

}
