package org.wso2.carbon.user.core.hybrid;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.def.DefaultUserStoreReader;
import org.wso2.carbon.user.core.def.SQLHelper;

public class HybridUserStoreReader implements UserStoreReader{
    
    protected UserStoreReader externalReader = null;
    private DefaultUserStoreReader defReader = null;
    protected SQLHelper sqlHelper = null;
    
    public HybridUserStoreReader(DataSource dataSource, UserStoreReader reader){
        externalReader = reader;
        sqlHelper = new HybridSQLHelper();
        defReader = new DefaultUserStoreReader(dataSource, sqlHelper);
    }
    
    public String[] getAllRoleNames() throws UserStoreException {
       String[] roleNames = externalReader.getAllRoleNames();
       String[] internalRoles = defReader.getAllRoleNames();
       if(internalRoles.length>0){
           List<String> totalRoles = new ArrayList<String>();
           for(String name : roleNames){
               totalRoles.add(name);
           }
           for(String name : internalRoles){
               totalRoles.add(name);
           }
           roleNames = totalRoles.toArray(new String[totalRoles.size()]);
       }
       return roleNames;
    }

    public String[] getAllUserNames() throws UserStoreException {
        String[] userNames = externalReader.getAllUserNames();
        return userNames;
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        String[] roleNames = externalReader.getAllRoleNames();
        
        for(String name : roleNames){
            if(name.equals(roleName)){
                return true;
            }
        }
        
        roleNames = defReader.getAllRoleNames();
        for(String name : roleNames){
            if(name.equals(roleName)){
                return true;
            }
        }
                
        return false;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        boolean isExistingUser = externalReader.isExistingUser(userName);
        return isExistingUser;
    }


    public String[] getUserRoles(String userName) throws UserStoreException {
        String[] userNames = externalReader.getAllUserNames();
        
        for(String name : userNames){
            if(name.equals(userName)){
                return externalReader.getUserRoles(userName);
            }
        }
        
        return new String[0];
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        String[] roleNames = externalReader.getAllRoleNames();
        
        for(String name : roleNames){
            if(name.equals(roleName)){
                return externalReader.getUsersInRole(roleName);
            }
        }
        
        roleNames = defReader.getAllRoleNames();
        for(String name : roleNames){
            if(name.equals(roleName)){
                return defReader.getUsersInRole(roleName);
            }
        }
                
        return new String[0];
        
    }

    public String[] listUsers(String filter, int maxItemLimit) 
    	throws UserStoreException{
	return externalReader.listUsers(filter, maxItemLimit);
    }
////// ***************************** Not going to implement ************************    
    
    public Map<String, String> getRoleProperties(String roleName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public List<String> getUserProfileNames(String userName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Map<String, String> getUserProperties(String userName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Map<String, String> getUserProperties(String userName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getUserPropertyNames() throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public boolean isExistingUserProfile(String userName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return false;
    }
    
    public String getDefaultUserProfileName(String userName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }
    



}
