package org.wso2.carbon.user.core.hybrid.actdir;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.BasicControl;
import javax.naming.ldap.Control;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.PagedResultsControl;
import javax.naming.ldap.PagedResultsResponseControl;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.i18n.Messages;

public class ActDirUserStoreReader implements UserStoreReader {

    protected DataSource dataSource = null;

    private static Log log = LogFactory.getLog(ActDirUserStoreReader.class);

    private ActDirRealmConfig config = null;

    public ActDirUserStoreReader(ActDirRealmConfig config) {
	this.config = config;
    }

    public String[] getAllRoleNames() throws UserStoreException {
	log.debug("Retrieving roles begin at ::"+System.currentTimeMillis());
	String[] roleNames = new String[0];
	SearchControls searchCtls = new SearchControls();
	searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);

	String searchFilter = "(objectcategory=group)";
	//(objectcategory=group)(objectClass=group)
	String searchBase = config.getSearchBase();

	try {
	    NamingEnumeration answer = config.getContext().search(searchBase, searchFilter,
		    searchCtls);
	    List<String> list = new ArrayList<String>();
	    while (answer.hasMoreElements()) {
		SearchResult sr = (SearchResult) answer.next();
		String name = sr.getName().substring(3);
		int index = name.indexOf(",");
		name = name.substring(0, index);
		list.add(name);
	    }
	    
	      
	    roleNames = list.toArray(new String[list.size()]);
	} catch (NamingException e) {
	    log.error("Problem getting roleNames: " + e);
	    // do not throw the exception. system should function properly
	}
	log.debug("Retrieving roles end at ::"+System.currentTimeMillis());
	return roleNames;
    }

    public String[] getAllUserNames() throws UserStoreException {
	String[] userNames = new String[0];
	SearchControls searchCtls = new SearchControls();
	searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);

	String searchFilter = "(&(&(objectCategory=person)(objectClass=user)(!(objectClass=computer))))";
	// String searchFilter = "(&(objectClass=user)(objectCategory=Andrew
	// Anderson))";
	String searchBase = config.getSearchBase();

	String returnedAtts[] = { "sAMAccountName" };
	searchCtls.setReturningAttributes(returnedAtts);
	try {
	    NamingEnumeration answer = config.getContext().search(searchBase, searchFilter,
		    searchCtls);
	    List<String> list = new ArrayList<String>();
	    while (answer.hasMoreElements()) {
		SearchResult sr = (SearchResult) answer.next();
		String name = (String) sr.getAttributes().get("sAMAccountName").get();
		list.add(name);
	    }
	    userNames = list.toArray(new String[list.size()]);
	} catch (NamingException e) {
	    log.error("Problem getting usernames: " + e);
	    // do not throw the exception. system should function properly
	}
	return userNames;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
	boolean isExistingUser = false;
	SearchControls searchCtls = new SearchControls();
	searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);        
	String searchFilter = "(&(objectCategory=person)(sAMAccountName="+userName+"))";
	// String searchFilter = "(&(objectClass=user)(objectCategory=Andrew
	// Anderson))";
	String searchBase = config.getSearchBase();

	String returnedAtts[] = { "sAMAccountName" };
	searchCtls.setReturningAttributes(returnedAtts);
	try {
	    NamingEnumeration answer = config.getContext().search(searchBase, searchFilter,
		    searchCtls);
	    List<String> list = new ArrayList<String>();
	    while (answer.hasMoreElements()) {
		isExistingUser = true;
		break;
	    }
	    
	} catch (NamingException e) {
	    log.error("Problem getting usernames: " + e);
	    // do not throw the exception. system should function properly
	}
	return isExistingUser;
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
	String[] userRoles = new String[0];
	SearchControls searchCtls = new SearchControls();
	searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);

	String searchFilter = "(&(objectCategory=person)(sAMAccountName=" + userName
		+ "))";
	String searchBase = config.getSearchBase();

	int totalResults = 0;
	String returnedAtts[] = { "memberOf" };

	try {
	    searchCtls.setReturningAttributes(returnedAtts);
	    NamingEnumeration answer = config.getContext().search(searchBase, searchFilter,
		    searchCtls);
	    int count = 0;
	    List<String> list = new ArrayList<String>();
	    while (answer.hasMoreElements()) {
		SearchResult sr = (SearchResult) answer.next();

		if (count > 0) {
		    log.error("More than one user exist with name :" + userName);
		}
		count++;
		// Print out the groups

		Attributes attrs = sr.getAttributes();
		if (attrs != null) {
		    try {
			for (NamingEnumeration ae = attrs.getAll(); ae.hasMore();) {
			    Attribute attr = (Attribute) ae.next();
			    for (NamingEnumeration e = attr.getAll(); e.hasMore(); totalResults++) {
				String value = e.next().toString();
				int index = value.indexOf(",");
				value = value.substring(3, index);
				list.add(value);
			    }

			}
		    } catch (NamingException e) {
			System.err.println("Problem listing user roles: " + e);
		    }

		}
	    }
	    userRoles = list.toArray(new String[list.size()]);
	} catch (NamingException e) {
	    log.error("Problem listing user roles: " + e);
	    // do not throw the exception. system should function properly
	}
	return userRoles;
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
	String[] userNames = new String[0];
	try {

	    SearchControls searchCtls = new SearchControls();
	    searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
	    String searchFilter = "(&((objectcategory=group))(CN=" + roleName + "))";
	    String searchBase = config.getSearchBase();

	    int totalResults = 0;

	    String returnedAtts[] = { "member" };
	    searchCtls.setReturningAttributes(returnedAtts);

	    NamingEnumeration answer = config.getContext().search(searchBase, searchFilter,
		    searchCtls);
	    int count = 0;
	    List<String> list = new ArrayList<String>();
	    while (answer.hasMoreElements()) {
		SearchResult sr = (SearchResult) answer.next();

		if (count > 0) {
		    log.error("More than one user exist with name :" + roleName);
		}
		count++;

		// Print out the members
		Attributes attrs = sr.getAttributes();

		if (attrs != null) {

		    try {
			for (NamingEnumeration ae = attrs.getAll(); ae.hasMore();) {
			    Attribute attr = (Attribute) ae.next();
			    for (NamingEnumeration e = attr.getAll(); e.hasMore(); totalResults++) {
				String value = e.next().toString();
				int index = value.indexOf(",");
				value = value.substring(3, index);
				value = ActDirUtil.getUserLoginName(config, value);
				list.add(value);
			    }
			}
		    } catch (NamingException e) {
			log.error("Problem listing members: " + e);
			throw new UserStoreException(Messages
				.getMessage("errorReadingFromUserStore"), e);
		    }

		}
	    }

	    userNames = list.toArray(new String[list.size()]);
	} catch (NamingException e) {
	    log.error("Problem searching directory: " + e);
	    // do not throw the exception. system should function properly
	}
	return userNames;
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
	boolean bFound = false;
	String[] names = this.getAllRoleNames();
	for (String name : names) {
	    if (name.equals(roleName)) {
		bFound = true;
		break;
	    }
	}
	return bFound;
    }

    // ////***************************** Not going to implement
    // ************************
    public Map getUserProperties(String userName) throws UserStoreException {
	return null;
    }

    public String[] getUserPropertyNames() throws UserStoreException {
	return null;
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
	    throws UserStoreException {
	return null;
    }

    public Map getRoleProperties(String roleName) throws UserStoreException {
	return new HashMap();
    }

    public Map<String, String> getUserProperties(String userName, String profileName)
	    throws UserStoreException {
	// TODO 1.0
	return null;
    }

    public List<String> getUserProfileNames(String userName) throws UserStoreException {
	// TODO 1.0
	return null;
    }

    public String getDefaultUserProfileName(String userName) throws UserStoreException {
	// TODO 1.0
	return null;
    }

    public boolean isExistingUserProfile(String userName, String profileName)
	    throws UserStoreException {
	// TODO 1.0
	return false;
    }

    public String[] listUsers(String filter, int maxItemLimit) 
    					throws UserStoreException{
	String[] userNames = new String[0];
	SearchControls searchCtls = new SearchControls();
	searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
	searchCtls.setCountLimit(maxItemLimit);
	
	if(filter.contains("?")){
	    throw new UserStoreException("invalid character for act dir serch");
	}
        
	String searchFilter = "(&(&(&(objectCategory=person)(objectClass=user)(!(objectClass=computer))(sAMAccountName="+filter+"))))";
	// String searchFilter = "(&(objectClass=user)(objectCategory=Andrew
	// Anderson))";
	String searchBase = config.getSearchBase();

	String returnedAtts[] = { "sAMAccountName" };
	searchCtls.setReturningAttributes(returnedAtts);
	try {
	    NamingEnumeration answer = config.getContext().search(searchBase, searchFilter,
		    searchCtls);
	    List<String> list = new ArrayList<String>();
	    int i = 0; 
	    while (answer.hasMoreElements() && i<maxItemLimit) {
		SearchResult sr = (SearchResult) answer.next();
		String name = (String) sr.getAttributes().get("sAMAccountName").get();
		list.add(name);
		i++;
	    }
	    userNames = list.toArray(new String[list.size()]);
	} catch (NamingException e) {
	    log.error("Problem getting usernames: " + e);
	    // do not throw the exception. system should function properly
	}
	return userNames;
    }


}
